program cc3cc2;                 (* converting CC3-map.txt files to CC2-format *)

       const TABCHAR = #9;
             LFCHAR = #10; (* CC2 files have lines LF delimited *)
             CRCHAR = #13; (* CC3-CC5 files have lines CR-LF delimited *)
       type string255 = string(.255.);


(* sub-programs *)

       function wortTAB(zeile : string255; position : integer) : string255;
       (* letzte Aenderung: 16.08.1986 *)
       var i,j : integer;
           erg : string255;

       begin
          i:=1;
          for j:=1 to position do
          begin
            while ((zeile[i]=LFCHAR) or (zeile[i]=TABCHAR)) and (i <= Length(zeile)) do i:=Succ(i);
            if (i > Length(zeile)) then wortTAB:=''
            else
            begin
              erg:=zeile[i];
              i:=Succ(i);
              while ((zeile[i]<>LFCHAR) and (zeile[i]<>TABCHAR)) and (i <= Length(zeile)) do
              begin
                erg:=erg+zeile[i];
                i:=Succ(i);
              end;
              wortTAB:=erg
            end
          end
       end;   (*   wort(..) : string255    *)


       (* Konvert ASCII to Integer and Integer to ASCII *)
       (* letzte Aenderung: 27.07.1986 --- Manfred Fischer *)

       function Atoi(kette : string255) : integer;
       CONST ascii_of_0 = 48;
       VAR i,sign,lager : Integer;

       begin
         lager:=0;
         sign := +1;
         for i:=1 to length(kette) do
           case kette[i] of
             '0'..'9' : lager := (lager*10) + (ord(kette[i])-ascii_of_0);
             '-' : sign := -1;
             else ; (* lager:=(lager*10) + 0 *)
           end;
         Atoi := sign * lager
       end;         (** atoi() **)


       function Itoa(zahl : integer) : string255;
       CONST ascii_of_0 = 48;
       VAR zwisch,rest : integer;
           erg : string255;

       begin
         erg:='';
         zwisch:=abs(zahl);
         repeat
           rest:=(zwisch mod 10);
           zwisch:=(zwisch div 10);
           erg := (chr(ascii_of_0 + rest)) + erg
         until (zwisch < 1);
         if (zahl < 0) then Itoa := '-' + erg
         else Itoa := erg;
       end;


       function RestOfLine(s : string255; delimiter : char) : string255;
       var result : string255;
           i : integer;
       begin
         result := '';
         i := 1;
         while (i <= length(s)) and (s[i] <> delimiter) do i := Succ(i);
         while (i <= length(s)) do
         begin
           result := result + s[i];
           i := Succ(i);
         end;
         RestOfLine := result;    (* includes delimiter! *)
       end;

       procedure ReadLine(var InHandle : text; Format : integer; var s : string255);
       const linelengthlimit = 255;
       var i : integer; c : char; lineend : boolean;
       begin
         (* read the line from file until delimiter is reached, read delimiter *)
         (* and transfer the chars (except the delimiter(s)) to var s *)
         (* i/o-check done ! *)
         i := 0;
         s := '';
         lineend := FALSE;
         repeat;
           lineend := eof(InHandle);
           if not lineend then
           begin
             read(InHandle, c);
             if (c = LFCHAR) then
             begin
               if (Format = 3) then lineend := TRUE
             end
             else if (c = CRCHAR) then
             begin
               if (Format = 2) then lineend := TRUE
             end
             else (* valid data *) if (i < linelengthlimit) then
             begin
               i := Succ(i);
               s := s + c;
             end;
           end;
         until lineend;
       end;

       procedure WriteLine(var OutHandle : text; Format : integer; var s : string255);
       begin
         if (Format = 2) then write(OutHandle, s, CRCHAR)
         else write(OutHandle, s, CRCHAR, LFCHAR);
       end;


const MaxTranstable = 1000;

type TranslationRecord = Record
                           name : string(.20.);
                           CC3value : integer;
                           CC2value : integer;
                           newname : string(.20.);
                           count : integer;
                         end;

var s, header, inputfilename, outputfilename : string255;
    InfileFormat : integer;               (* determines CloseCombat format *)
    OutfileFormat : integer;

    infile, outfile : Text;      (* file handles *)

    translation : array(.0..MaxTranstable.) of TranslationRecord;
    i, transmin, transmax : integer;
    linecounter : integer;
    value : integer;

begin
   OutfileFormat := 2; (* CC2 as default *)
   InfileFormat := 3;  (* CC3 as default *)

   writeln('CC3-CC2 - a CloseCombat map.txt-file converter by Mafi');
   writeln('written in TurboPascal - sent comments to closecombat2@claranet.de');
   writeln('be carefull - no I/O-checking is done');
   writeln('the input file must reside in the same folder as the program');
   writeln('input format will be:  CC3');
   write('output format will be: CC2');

   writeln;
   writeln('this little program will convert a CC-map.txt file to CC2-format');
   writeln('but not the graphics (use one of TinTins TextureMaker tools),');
   writeln('only the terrain-elements and -elevation definitions are handled.');

   writeln;
   writeln('for correct translation it uses an external translation table,');
   write('enter filename of translation table: ');
   readln(inputfilename);

   (* reading translation table *)

   Assign(infile, inputfilename);
   Reset(infile);
   readln(infile, s);
   writeln('using ',s);
   s := ' ';
   transmin := 0;
   transmax := 0;
   while (not eof(infile)) and (s(.1.) <> '&') do readln(infile, s);
   while (not eof(infile)) and (s(.1.) <> '#') do
   begin
     readln(infile, s);
     if (transmax <= MaxTranstable) and (s(.1.) <> '#') then
     begin
       with translation(.transmax.) do
       begin
         name := wortTAB(s,1);
         CC3value := atoi(wortTAB(s,2));
         count := 0;
         if (CC3value <> transmax) then
         begin
           writeln('error in translation-table, value ',CC3value);
           writeln('wrong sequence of CC3-values - aborting.');
           Exit;
         end;
         CC2value := atoi(wortTAB(s,3));
         if (CC2value = 0) and (transmax <> 0) then writeln('warning: perhaps definition missing CC3-value ',CC3value);
         newname := wortTAB(s,4);
       end;
       transmax := Succ(transmax);
     end;
   end;
   transmax := Pred(transmax);

   if (transmax < transmin) then
   begin
     writeln('error: no definitions in file - aborting');
     Exit;
   end;

   Close(infile);
   writeln(Succ(transmax),' translation definitions found.');
   writeln;

   (* now looking for the input file *)
   (* and setting up the new output file *)

   writeln('now the CC3/CC4/CC5 *****.txt file can be converted to CC2-format,');
   writeln('it must reside in the same folder as this program.');
   write('enter file name: ');
   readln(inputfilename);

   if (OutfileFormat = 2) then outputfilename := 'MAP-CC2'
   else outputfilename := 'NEW-CC3.txt';

   writeln('resulting output will be written into file ',outputfilename);

   Assign(infile, inputfilename);
   Reset(infile); (* open file for input *)
   Assign(outfile, outputfilename);
   Rewrite(outfile); (* create new file for output *)

   (* first: checking the header informations of input file *)

   readln(infile, header);

   if (header(.1.) = '1') and (header(.2.) = '0') then
   begin
       write('valid CC3-header found');
       InfileFormat := 3;
   end
   else if (header(.1.) = '4') and (header(.2.) = '1') then
   begin
       write('valid CC2-header found');
       InfileFormat := 2;
   end
   else
   begin
       writeln('aborting: no CC3-header: ', header);
       Close(infile);
       Close(outfile);
       Exit;
   end;

   Close(Infile);
   Reset(Infile); (* read again *)

   (* now start reading the lines *)

   ReadLine(Infile, InfileFormat, header); (* header line 1 *)
   if (OutfileFormat = 2) then
   begin
     header(.1.) := '4';
     header(.2.) := '1';
     WriteLine(outfile, OutfileFormat, header);
   end
   else if (OutfileFormat = 3) then
   begin
     header(.1.) := '1';
     header(.2.) := '0';
     WriteLine(outfile, OutfileFormat, header);
     s := ''; (* can contain name of map *)
     WriteLine(outfile, OutfileFormat, s);
   end;
   writeln(' ... and written');

   if (InfileFormat = 3) then
   begin
     ReadLine(Infile, InfileFormat, s); (* header line 2 *)
   end; (* CC2 has no second header line *)


   (* second: take the original file datas *)
   (* valid for all file formats *)

   ReadLine(Infile, InfileFormat, s); (* maxx *)
   WriteLine(outfile, OutfileFormat, s);
   writeln(s);     (* echo to console *)
   ReadLine(Infile, InfileFormat, s); (* maxy *)
   WriteLine(outfile, OutfileFormat, s);
   writeln(s);     (* echo to console *)

   (* third: skipping next two CC3-lines *)

   if (InfileFormat = 3) then
   begin
     ReadLine(Infile, InfileFormat, s); (* header line 5 *)
     ReadLine(Infile, InfileFormat, s); (* header line 6 *)
   end; (* CC2 has no second header line *)

   if (OutfileFormat = 3) then
   begin
     s := '0';
     WriteLine(Outfile, OutfileFormat, s);
     WriteLine(Outfile, OutfileFormat, s);
   end;

   (* forth: take the original comment lines *)

   ReadLine(Infile, InfileFormat, s); (* header line 7 *)
   WriteLine(Outfile, OutfileFormat, s);

   (* fifth: looking for data start line *)

   repeat
     write('&');
     ReadLine(Infile, InfileFormat, s); (* header line 8 *)
   until (s(.1.) = '&');
   WriteLine(Outfile, OutfileFormat, s);

   (* reading first data line *)

   ReadLine(Infile, InfileFormat, s); (* header line 8 *)
   write('processing line ',' ':5);
   linecounter := 1;

   (* sixth: reading the data lines and writing them translated *)
   (*        including the change of the numbering in first column *)
   (* valid for all file formats: searching for the end-line "#" *)

   while (not eof(infile)) and (s(.1.) <> '#') do
   begin
     write(#8#8#8#8#8,linecounter:5);
     write(outfile, wortTAB(s,1)); (* numbering *)
     for i := 2 to 17 do        (* 16 terrain values *)
     begin
       value := atoi(wortTAB(s,i));
       if (value >= transmin) and (value <= transmax) then
       begin
         write(outfile, TABCHAR, Translation(.value.).CC2value);
         with Translation(.value.) do
           if CC2value <> CC3value then count := 1;
       end
       else
       begin
         writeln('invalid input data: ',value,' translated to 84 = wreck');
         value := 84; (* wreck *)
         write(outfile, TABCHAR, value);
       end;
     end;
     write(outfile, TABCHAR, wortTAB(s, 18)); (* elevation *)
     s := '';
     WriteLine(outfile, OutfileFormat, s); (* end of line *)
     ReadLine(infile, InfileFormat, s);
     linecounter := Succ(linecounter);
   end;

   writeln;
   (* end of datas *)

   if (s(.1.) = '#') then (* end of datas *)
   begin
     WriteLine(outfile, OutfileFormat, s);
     if (not eof(infile)) then
     begin
       ReadLine(infile, InfileFormat, s);
       if (length(s) >= 1) then
       begin
         WriteLine(outfile, OutfileFormat, s);
         writeln('additional datas behind first "#" found, will be written without change');
         while not eof(infile) do
         begin
           ReadLine(infile, InfileFormat, s);
           WriteLine(outfile, OutfileFormat, s);
         end;
         write('end of input file...');
       end;
     end;
   end
   else
   begin
     writeln('unexpected end of input file');
     s := '#';
     WriteLine(outfile, OutfileFormat, s);
   end;

   (* done *)

   Close(infile);
   Close(outfile);
   writeln('job done, result in ', outputfilename);
   writeln('dont forget to control the elevation values,');
   writeln('shrinking the map can be done with CCshrink.exe');

   write('statistics (yes/no) ? ');
   readln(s);
   if (length(s) = 0) then (* nothing *)
   else if (s(.1.) = 'n') or (s(.1.) = 'N') then (* no *)
   else
   begin
     for i := transmin to transmax do
     begin
       with Translation(.i.) do if (count > 0) then
       begin
         writeln(name,' (',CC3value,') - ',CC2value,' ',newname);
       end;
     end;
   end;
end.


